/*
<DEPENDENCIES>
    <DEPENDS>Browser</DEPENDS>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Snapshot</DEPENDS>
    <DEPENDS>Delta</DEPENDS>
    <DEPENDS>AlterableWidget</DEPENDS>
</DEPENDENCIES>
*/
function MouseMoveHandle( iMouseDownX, iMouseDownY )
{
  this.moving = false;
  this.downX = iMouseDownX;
  this.downY = iMouseDownY;
}
MouseMoveHandle.prototype.isMoving = function()
{
  return this.moving;
}
MouseMoveHandle.prototype.startMoving = function()
{
  this.moving = true;
}
MouseMoveHandle.prototype.stopMoving = function()
{
  this.moving = false;
}
function AlterationStrategy( sId, sWidgetDomId, bConverge )
{
  if( sId )
  {
    this.init(sId, sWidgetDomId, bConverge);
  }
}
AlterationStrategy.prototype.init = function( sId, sWidgetDomId, bConverge )
{
  ;
  this.sId = sId;
  this.widgetDomId = sWidgetDomId;
  this.nWidget = document.getElementById(sWidgetDomId);
  this.shouldConverge = bConverge;
  this.aAlterables = new Array();
  this.aSnapshots = new Array();
}
AlterationStrategy.prototype.getId = function()
{
  return this.sId;
}
AlterationStrategy.prototype.registerAlterable = function( alterable )
{
  if( alterable )
    this.aAlterables.push(alterable);
}
AlterationStrategy.prototype.snapshot = function()
{
  for( var i = 0; i < this.aAlterables.length; i++ )
  {
    this.aSnapshots.push(this.aAlterables[i].createSnapshot());
  }
}
AlterationStrategy.prototype.forgetSnapshots = function()
{
  this.aSnapshots.length = 0;
}
AlterationStrategy.prototype.isFeasible = function( delta )
{
  for( var i = 0; i < this.aSnapshots.length; i++ )
  {
    var snapshot = this.aSnapshots[i];
    if( snapshot.parent.isFeasible(snapshot, delta) == false )
      return false; 
  }
  return true;
}
AlterationStrategy.prototype.getFeasibleDelta = function( delta )
{
  var feasibleDelta = delta;
  for( var i = 0; i < this.aSnapshots.length; i++ )
  {
    var snapshot = this.aSnapshots[i];
    feasibleDelta = snapshot.parent.getFeasibleDelta(snapshot, feasibleDelta);
  }
  return feasibleDelta;
}
AlterationStrategy.prototype.alter = function( delta )
{
  if( this.isFeasible(delta) )
  {
    for( var i = 0; i < this.aSnapshots.length; i++ )
    {
      var snapshot = this.aSnapshots[i];
      snapshot.parent.alter(snapshot, delta);
    }
  }
  else if( this.shouldConverge )
  {
    var feasibleDelta = this.getFeasibleDelta(delta);
    for( var i = 0; i < this.aSnapshots.length; i++ )
    {
      var snapshot = this.aSnapshots[i];
      snapshot.parent.alter(snapshot, feasibleDelta);
    }
  }
}
AlterationStrategy.prototype.reset = function()
{
  for( var i = 0; i < this.aSnapshots.length; i++ )
  {
    var snapshot = this.aSnapshots[i];
    snapshot.parent.reset(snapshot);
  }
}
AlterationStrategy.createMovingMouseDownEventHandler = function( me )
{
  me.mouseDownHandler = function( event )
  {
    ;
    if( !event )
      event = window.event;
    Browser.removeEventListenerFromDomNode(me.nWidget, Browser.eventNames.MOUSEDOWN, me.mouseDownHandler);
    Browser.eventStopPropagation(event);
    Browser.eventPreventDefault(event);
    me.mouseMove = new MouseMoveHandle(event.clientX, event.clientY);
    Browser.addEventListenerToDocument(Browser.eventNames.MOUSEUP, me.mouseUpHandler);
    Browser.addEventListenerToDocument(Browser.eventNames.MOUSEMOVE, me.mouseMoveHandler);
    return false;
  }
}
AlterationStrategy.createMovingMouseUpEventHandler = function( me )
{
  me.mouseUpHandler = function( event )
  {
    ;
    if( !event )
      event = window.event;
    Browser.removeEventListenerFromDocument(Browser.eventNames.MOUSEMOVE, me.mouseMoveHandler);
    Browser.removeEventListenerFromDocument(Browser.eventNames.MOUSEUP, me.mouseUpHandler);
    me.mouseMove.stopMoving();
    me.stopMoving(me.mouseMove, event);
    me.mouseMove = undefined;
    Browser.addEventListenerToDomNode(me.nWidget, Browser.eventNames.MOUSEDOWN, me.mouseDownHandler);
  }
}
AlterationStrategy.createMovingMouseMoveEventHandler = function( me )
{
  me.mouseMoveHandler = function( event )
  {
    ;
    if( !event )
      event = window.event;
    if( !me.mouseMove.isMoving() && me.isThresholdExceeded(me.mouseMove, event) )
    {
      me.startMoving(me.mouseMove, event);
      me.mouseMove.startMoving();
    }
    if( me.mouseMove.isMoving() )
    {
      me.doMove(me.mouseMove, event);
    }
  }
}
AlterationStrategy.createMouseClickEventHandler = function( me )
{
  me.mouseClickHandler = function( event )
  {
    ;
    if( !event )
      event = window.event;
    me.doStep(event);
  }
}
AlterationStrategy.isThresholdExceeded = function( currentPos, initPos, threshold )
{
  if( !threshold )
  {
    return true;
  }
  else
  {
    return Math.abs(currentPos - initPos) > threshold;
  }
}
ContinuousAlterationStrategy.prototype = new AlterationStrategy();
ContinuousAlterationStrategy.prototype.constructor = ContinuousAlterationStrategy;
ContinuousAlterationStrategy.superclass = AlterationStrategy.prototype;
function ContinuousAlterationStrategy( sId, sWidgetDomId, bConverge, iHorizontalThreshold, iVerticalThreshold )
{
  if( sId )
  {
    this.init(sId, sWidgetDomId, bConverge, iHorizontalThreshold, iVerticalThreshold);
  }
}
ContinuousAlterationStrategy.prototype.init = function( sId, sWidgetDomId, bConverge, iHorizontalThreshold,
                                                        iVerticalThreshold )
{
  ;
  ContinuousAlterationStrategy.superclass.init.call(this, sId, sWidgetDomId, bConverge);
  this.horizontalThreshold = iHorizontalThreshold;
  this.verticalThreshold = iVerticalThreshold;
  AlterationStrategy.createMovingMouseDownEventHandler(this);
  AlterationStrategy.createMovingMouseUpEventHandler(this);
  AlterationStrategy.createMovingMouseMoveEventHandler(this);
  Browser.addEventListenerToDomNode(this.nWidget, Browser.eventNames.MOUSEDOWN, this.mouseDownHandler);
}
ContinuousAlterationStrategy.prototype.isThresholdExceeded = function( mouseMoveHandle, event )
{
  return AlterationStrategy.isThresholdExceeded(event.clientX, mouseMoveHandle.downX, this.horizontalThreshold)
          || AlterationStrategy.isThresholdExceeded(event.clientY, mouseMoveHandle.downY, this.verticalThreshold);
}
ContinuousAlterationStrategy.prototype.startMoving = function( mouseMoveHandle, event )
{
  this.snapshot();
}
ContinuousAlterationStrategy.prototype.stopMoving = function( mouseMoveHandle, event )
{
  this.forgetSnapshots();
}
ContinuousAlterationStrategy.prototype.doMove = function( mouseMoveHandle, event )
{
  this.alter(new Delta(event.clientX - mouseMoveHandle.downX, event.clientY - mouseMoveHandle.downY));
}
SteppingAlterationStrategy.prototype = new AlterationStrategy();
SteppingAlterationStrategy.prototype.constructor = SteppingAlterationStrategy;
SteppingAlterationStrategy.superclass = AlterationStrategy.prototype;
function SteppingAlterationStrategy( sId, sWidgetDomId, bConverge, iHorizontalStepWidth, iVerticalStepWidth,
                                     iHorizontalThreshold, iVerticalThreshold )
{
  if( sId )
  {
    this.init(sId, sWidgetDomId, bConverge, iHorizontalStepWidth, iVerticalStepWidth, iHorizontalThreshold, iVerticalThreshold);
  }
}
SteppingAlterationStrategy.prototype.init = function( sId, sWidgetDomId, bConverge, iHorizontalStepWidth,
                                                      iVerticalStepWidth,
                                                      iHorizontalThreshold, iVerticalThreshold )
{
  ;
  SteppingAlterationStrategy.superclass.init.call(this, sId, sWidgetDomId, bConverge);
  this.horizontalStepWidth = iHorizontalStepWidth;
  this.verticalStepWidth = iVerticalStepWidth;
  this.horizontalThreshold = iHorizontalThreshold;
  this.verticalThreshold = iVerticalThreshold;
  AlterationStrategy.createMovingMouseDownEventHandler(this);
  AlterationStrategy.createMovingMouseUpEventHandler(this);
  AlterationStrategy.createMovingMouseMoveEventHandler(this);
  Browser.addEventListenerToDomNode(this.nWidget, Browser.eventNames.MOUSEDOWN, this.mouseDownHandler);
}
SteppingAlterationStrategy.prototype.isThresholdExceeded = function( mouseMoveHandle, event )
{
  return AlterationStrategy.isThresholdExceeded(event.clientX, mouseMoveHandle.downX, this.threshold)
          || AlterationStrategy.isThresholdExceeded(event.clientY, mouseMoveHandle.downY, this.threshold);
}
SteppingAlterationStrategy.prototype.startMoving = function( mouseMoveHandle, event )
{
  ;
  this.snapshot();
}
SteppingAlterationStrategy.prototype.stopMoving = function( mouseMoveHandle, event )
{
  if( this.horizontalStepWidth || this.verticalStepWidth )
  {
    var delta = new Delta(
            SteppingAlterationStrategy.calculateNearest(event.clientX
                    - mouseMoveHandle.downX, this.horizontalStepWidth),
            SteppingAlterationStrategy.calculateNearest(event.clientY - mouseMoveHandle.downY, this.verticalStepWidth));
    this.alter(delta);
  }
  this.forgetSnapshots();
}
SteppingAlterationStrategy.prototype.doMove = function( mouseMoveHandle, event )
{
  var delta = new Delta(event.clientX - mouseMoveHandle.downX, event.clientY - mouseMoveHandle.downY);
  this.alter(delta);
}
SteppingAlterationStrategy.calculateNearest = function( iDelta, iStepwidth )
{
  if( iStepwidth )
  {
    var low = Math.floor(iDelta / iStepWidth) * iStepWidth;
    var high = Math.ceil(iDelta / iStepWidth) * iStepWidth;
    if( iDelta - low < high - iDelta )
      return low;
    else
      return high;
  }
  else
  {
    return 0;
  }
}
ExpandStepAlterationStrategy.prototype = new AlterationStrategy();
ExpandStepAlterationStrategy.prototype.constructor = ExpandStepAlterationStrategy;
ExpandStepAlterationStrategy.superclass = AlterationStrategy.prototype;
function ExpandStepAlterationStrategy( sId, sWidgetDomId, bConverge, iHorizontalStepWidth, iVerticalStepWidth )
{
  if( sId )
  {
    this.init(sId, sWidgetDomId, bConverge, iHorizontalStepWidth, iVerticalStepWidth);
  }
}
ExpandStepAlterationStrategy.prototype.init = function( sId, sWidgetDomId, bConverge, iHorizontalStepWidth,
                                                        iVerticalStepWidth )
{
  ;
  ExpandStepAlterationStrategy.superclass.init.call(this, sId, sWidgetDomId, bConverge);
  this.horizontalStepWidth = iHorizontalStepWidth;
  this.verticalStepWidth = iVerticalStepWidth;
  AlterationStrategy.createMouseClickEventHandler(this);
  Browser.addEventListenerToDomNode(this.nWidget, Browser.eventNames.MOUSECLICK, this.mouseClickHandler);
}
ExpandStepAlterationStrategy.prototype.doStep = function( event )
{
  ;
  this.snapshot();
  this.alter(new Delta(this.horizontalStepWidth, this.verticalStepWidth));
  this.forgetSnapshots();
}
ShrinkStepAlterationStrategy.prototype = new AlterationStrategy();
ShrinkStepAlterationStrategy.prototype.constructor = ShrinkStepAlterationStrategy;
ShrinkStepAlterationStrategy.superclass = AlterationStrategy.prototype;
function ShrinkStepAlterationStrategy( sId, sWidgetDomId, bConverge, iHorizontalStepWidth, iVerticalStepWidth )
{
  if( sId )
  {
    this.init(sId, sWidgetDomId, bConverge, iHorizontalStepWidth, iVerticalStepWidth);
  }
}
ShrinkStepAlterationStrategy.prototype.init = function( sId, sWidgetDomId, bConverge, iHorizontalStepWidth,
                                                        iVerticalStepWidth )
{
  ;
  ShrinkStepAlterationStrategy.superclass.init.call(this, sId, sWidgetDomId, bConverge);
  this.horizontalStepWidth = iHorizontalStepWidth;
  this.verticalStepWidth = iVerticalStepWidth;
  AlterationStrategy.createMouseClickEventHandler(this);
  Browser.addEventListenerToDomNode(this.nWidget, Browser.eventNames.MOUSECLICK, this.mouseClickHandler);
}
ShrinkStepAlterationStrategy.prototype.doStep = function( event )
{
  ;
  this.snapshot();
  this.alter(new Delta(-this.horizontalStepWidth, -this.verticalStepWidth));
  this.forgetSnapshots();
}
