
/* /1175261643647_0.js

Logger.js
toolkit/staticexport/StructureEntry.js
domapi/Globals.js
common/broker/Broker.js
actions/Actions.js
toolkit/staticexport/PropertyService.js
toolkit/staticexport/LocationService.js
toolkit/staticexport/LMSSuspendDataSerializer.js
toolkit/staticexport/LMSConnectorManagerStandalone.js
toolkit/staticexport/ResultData.js
toolkit/staticexport/EvaluationEntry.js
toolkit/staticexport/EvaluationManager.js
toolkit/staticexport/StructureManager.js
toolkit/staticexport/LinkService.js
toolkit/staticexport/ScoreStatistics.js
toolkit/staticexport/ScoreEvaluatorStandard.js
toolkit/staticexport/ServiceRegistry.js
toolkit/staticexport/AchievementService.js
*/

/* Logger.js */
/*
<DEPENDENCIES>
</DEPENDENCIES>
*/
// Logger

/**
 * Defines the Logging class. This will usually be a single global instance
 *
 * @constructor
 */
function Logger()
{
	this.nFatal  = 0;
	this.nError =       this.nFatal + 1;
	this.nWarning =     this.nError + 1;
	this.nDebug =       this.nWarning + 1;
	this.nInfo =        this.nDebug + 1;
	this.nVerbose =     this.nInfo + 1;
	this.nTrace =       this.nVerbose + 1;
	this.nVeryVerbose = this.nTrace + 1;

	this.sDebugFrame = 'debugFrame';
	this.bActive = true;
	this.nLevel = this.nVeryVerbose;
	this.nLine = 0;
	this.sMask = '';

	this.aColor = new Array();
	this.aPrefix = new Array();
	this.aMask = new Array();

	this.aColor[this.nFatal]       = '#ff0000';
	this.aColor[this.nError]       = '#ff0000';
	this.aColor[this.nWarning]     = '#ff6666';
	this.aColor[this.nDebug]       = '#000000';
	this.aColor[this.nInfo]        = '#0000ff';
	this.aColor[this.nVerbose]     = '#588AB5';
	this.aColor[this.nTrace]       = '#785A95';
	this.aColor[this.nVeryVerbose] = '#006600';

	this.aPrefix[this.nFatal]       = 'Fatal&nbsp;&nbsp;';
	this.aPrefix[this.nError]       = 'Error&nbsp;&nbsp;';
	this.aPrefix[this.nWarning]     = 'Warning';
	this.aPrefix[this.nDebug]       = 'Debug&nbsp;&nbsp;';
	this.aPrefix[this.nInfo]        = 'Info&nbsp;&nbsp;&nbsp;';
	this.aPrefix[this.nVerbose]     = '.......';
	this.aPrefix[this.nTrace]       = 'Trace&nbsp;&nbsp;';
	this.aPrefix[this.nVeryVerbose] = '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';

  this.aHex = new Array();
  this.aHex[0] = '0';
  this.aHex[1] = '1';
  this.aHex[2] = '2';
  this.aHex[3] = '3';
  this.aHex[4] = '4';
  this.aHex[5] = '5';
  this.aHex[6] = '6';
  this.aHex[7] = '7';
  this.aHex[8] = '8';
  this.aHex[9] = '9';
  this.aHex[10] = 'a';
  this.aHex[11] = 'b';
  this.aHex[12] = 'c';
  this.aHex[13] = 'd';
  this.aHex[14] = 'e';
  this.aHex[15] = 'f';

  this.aModuleColor = new Array();
  
  this.lastLine = null; // for scrolling


  try
  {
  	if(!parent.frames[this.sDebugFrame])
  		this.bActive=false;
  }
  catch(e)
  {
  	this.bActive=false;
  }

  if ( this.bActive && parent.frames[this.sDebugFrame])
	{
		parent.frames[this.sDebugFrame].document.write('<html><head>\n');
		parent.frames[this.sDebugFrame].document.write('<link rel="stylesheet" href="../af-web/js/Logger.css" type="text/css">\n');
		parent.frames[this.sDebugFrame].document.write('</head><body>\n');
		parent.frames[this.sDebugFrame].document.write('<p class="Logger_Text">\n');
	}

  if ( this.bActive && parent.frames[this.sDebugFrame]) 
  {
    for (var i = this.nFatal; i <= this.nVeryVerbose; i++) 
    {
      this.aMask[i] = true;
    }
  }

	if ( this.bActive && parent.frames[this.sDebugFrame]) 
	{
		var sQuery = parent.document.location.search;
		sQuery = sQuery.substring(1, sQuery.length);
    var aArgs = sQuery.match(/([^=]+)\=([^&]+)/);

		if (aArgs != null)
		{
			for (var i = 1; i < aArgs.length; i++)
			{
				if (false) {} 
				else if (aArgs[i] == 'logmask')
				{
					i++;

          this.setLogMask(aArgs[i]);
				}
			}
		}
	}
	
}


/**
 * Set log mask from string. 
 *
 * @param sString mask string like "f,e,d,i,v,t,vv" or "-t,vv"
 */
Logger.prototype.setLogMask = function(sMask)
{
  var aName = new Array();
  aName[this.nFatal]       = 'f';
  aName[this.nError]       = 'e';
  aName[this.nWarning]     = 'w';
  aName[this.nDebug]       = 'd';
  aName[this.nInfo]        = 'i';
  aName[this.nVerbose]     = 'v';
  aName[this.nTrace]       = 't';
  aName[this.nVeryVerbose] = 'vv';

  this.sMask = sMask;

  for (var i = this.nFatal; i <= this.nVeryVerbose; i++) 
  {
    this.aMask[i] = true;
  }

  if (sMask == '')
  { 
    //
  }
  else 
  {
    var bValue = true;

    if (false) {}
    else if (sMask.substring(0, 1) == '-')
    {
      sMask = sMask.substring(1, sMask.length);
      bValue = false;
      for (var j = this.nFatal; j <= this.nVeryVerbose; j++) this.aMask[j] = true;
    }
    else
    {
      bValue = true;
      for (var j = this.nFatal; j <= this.nVeryVerbose; j++) this.aMask[j] = false;
    }

    aLogmask = sMask.split(',');
    for (var j = 0; j < aLogmask.length; j++)
    {
      var nLevel = -1;
      for (var k = this.nFatal; k <= this.nVeryVerbose; k++) 
      {
        if (aLogmask[j] == aName[k])
        {
          nLevel = k;
          break;
        }
      }
      if (nLevel >= 0) 
      {
        this.aMask[nLevel] = bValue;
      }
    }
  }
  
} 


/**
 * Return the log mask as string. 
 *
 */
Logger.prototype.getLogMask = function()
{
  var aName = new Array();
  aName[this.nFatal]       = 'f';
  aName[this.nError]       = 'e';
  aName[this.nWarning]     = 'w';
  aName[this.nDebug]       = 'd';
  aName[this.nInfo]        = 'i';
  aName[this.nVerbose]     = 'v';
  aName[this.nTrace]       = 't';
  aName[this.nVeryVerbose] = 'vv';
  
  var sMask = '';

  for (var i = this.nFatal; i <= this.nVeryVerbose; i++) 
  {
    if (this.aMask[i])
    {
      if (sMask != '') sMask += ',';
      sMask += aName[i];
    }
  }

  return sMask;
} 


Logger.prototype.getHex = function(nValue)
{
  var sValue = '';
  var nLow = nValue % 16;
  var nHigh = (nValue - nValue % 16) / 16;
  sValue = this.aHex[nHigh] + this.aHex[nLow];
  //alert(nValue+' '+nLow+' '+nHigh+' '+sValue);
  return sValue;
}

Logger.prototype.getModuleColor = function(sModule)
{
  if (this.aModuleColor[sModule] == null)
  {
    var sColor = '#000000';
    var nSum = 0;
    for (var i = 0; i < sModule.length; i++) 
    {
      var nChar = sModule.charCodeAt(i);
      nSum += parseInt(nChar);
    }
    var nRed = nSum % 171;
    var nGreen = nSum % 215;
    var nBlue = nSum % 191;
    this.aModuleColor[sModule] = '#' + this.getHex(nRed) + this.getHex(nGreen) + this.getHex(nBlue);
  }
  return this.aModuleColor[sModule];
}

/**
 * Generic print. 
 *
 * @param nLevel The debug level
 * @param sModule module name or class name that reports
 * @param sMessage log message, usually contains current METHOD and the actual message text
 */
Logger.prototype.print = function(nLevel, sModule, sMessage)
{
	if ( !this.bActive || !parent.frames[this.sDebugFrame]) return;

	if (nLevel <= this.nLevel && this.aMask[nLevel])
	{
		this.nLine++;
    var sColor = this.aColor[nLevel];
    
    var bBold = false;
    switch (nLevel)
    {
       case this.nFatal: bBold = true; break;
       case this.nError: bBold = true; break;
       //case this.nDebug: bBold = true; break;
      default: ;
    }
        
    if (!bBold) sColor = this.getModuleColor(sModule);
    
    switch (sModule)
    {
      case 'bla': sColor = '#123456'; break;
      default: ;
    }
    
    if (nLevel == this.nDebug)
    {
      sColor = '#0000ff';
    }
    
    var sOut = '<font id="Logger_' + (this.nLine) + '" color=\"' + sColor + '\">' + this.aPrefix[nLevel] + (bBold?'<b>':'') + ' [' + sModule + '] ' + sMessage + (bBold?'</b>':'') + '</font><br>\n';
		parent.frames[this.sDebugFrame].document.write(sOut);
		
    var o = parent.frames[this.sDebugFrame].document.getElementById('Logger_'+this.nLine);
    if (o != null) 
    {
      if (this.nLine % 20 == 0)
      {
        o.scrollIntoView(true);
        this.lastLine = null;
      }
      else
      {
        this.lastLine = o;
      }
    }
	}
  
} 


/**
 * Wrapper for fatal error messages. Fatal means program will not be able to continue.
 *
 * @param sModule see Logger.prototype.print
 * @param sMessage see Logger.prototype.print
 */
Logger.prototype.fatal = function(sModule, sMessage)
{
	if ( !this.bActive || !parent.frames[this.sDebugFrame]) return;
	this.print(this.nFatal, sModule, sMessage);
}



/**
 * Wrapper for error messages. Means something serious happens, but program will recover
 *
 * @param sModule see Logger.prototype.print
 * @param sMessage see Logger.prototype.print
 */
Logger.prototype.error = function(sModule, sMessage)
{
	if ( !this.bActive || !parent.frames[this.sDebugFrame]) return;
	this.print(this.nError, sModule, sMessage);
}


/**
 * Wrapper for warnings. A bad condition.
 *
 * @param sModule see Logger.prototype.print
 * @param sMessage see Logger.prototype.print
 */
Logger.prototype.warning = function(sModule, sMessage)
{
	if ( !this.bActive || !parent.frames[this.sDebugFrame]) return;
	this.print(this.nWarning, sModule, sMessage);
}


/**
 * Wrapper for debug output. Use this for temporary printf-debugging.
 *
 * @param sModule see Logger.prototype.print
 * @param sMessage see Logger.prototype.print
 */
Logger.prototype.debug = function(sModule, sMessage)
{
	if ( !this.bActive || !parent.frames[this.sDebugFrame]) return;
	this.print(this.nDebug, sModule, sMessage);
}


/**
 * Wrapper for info messages. Output important state changes. Expected to generate few lines per second
 *
 * @param sModule see Logger.prototype.print
 * @param sMessage see Logger.prototype.print
 */
Logger.prototype.info = function(sModule, sMessage)
{
	if ( !this.bActive || !parent.frames[this.sDebugFrame]) return;
	this.print(this.nInfo, sModule, sMessage);
}


/**
 * Wrapper for verbose output. Expected to generate 50 lines per second.
 *
 * @param sModule see Logger.prototype.print
 * @param sMessage see Logger.prototype.print
 */
Logger.prototype.verbose = function(sModule, sMessage)
{
	if ( !this.bActive || !parent.frames[this.sDebugFrame]) return;
	this.print(this.nVerbose, sModule, sMessage);
}


/**
 * Wrapper for method entry messages, optional exit messages.
 *
 * @param sModule see Logger.prototype.print
 * @param sMessage see Logger.prototype.print
 */
Logger.prototype.trace = function(sModule, sMessage)
{
	if ( !this.bActive || !parent.frames[this.sDebugFrame]) return;
		this.print(this.nTrace, sModule, sMessage);
}


/**
 * Wrapper for very verbose output. Expected to generate 1000 lines per second.
 *
 * @param sModule see Logger.prototype.print
 * @param sMessage see Logger.prototype.print
 */
Logger.prototype.veryverbose = function(sModule, sMessage)
{
	if ( !this.bActive || !parent.frames[this.sDebugFrame]) return;
		this.print(this.nVeryVerbose, sModule, sMessage);
}


Logger.prototype.scrollToLastLine = function()
{
  if (this.lastLine != null)
  {
    this.lastLine.scrollIntoView(true);
    this.lastLine = null;
  }
} 

Logger.prototype.objectTypeToString = function( object )
{
  if( object == undefined )
    return 'object is UNDEFINED';
  if( object == null )
    return 'object is NULL';

  return object.constructor;
}

Logger.prototype.objectToString = function( object, mode )
{
  if( object == undefined )
    return 'UNDEFINED';
  if( object == null )
    return 'NULL';
  // ---- doesn't work in IE.
  //  if( HTMLElement.prototype.isPrototypeOf(object) )
  //    return this.htmlElementToString(object);
  // ----
  if( object.tagName != undefined )
    return this.htmlElementToString(object);

  var result = '';

  if( mode == undefined || mode == null )
  {
    for( name in object )
    {
      if( result == '' )
        result += name;
      else
        result += ', ' + name;
    }

    if( result == '' )
      result = '{ -- NO ENUMERABLE PROPERTIES -- }';
    else
      result = '{ ' + result + ' }';
  }
  else if( mode == 'v' )
  {
    for( name in object )
    {
      result += name + '=' + object[name] + ';';
    }

    if( result == '' )
      result = '{ -- NO ENUMERABLE PROPERTIES -- }';
    else
      result = '{ ' + result + ' }';
  }
  else if( mode == 'vv' )
  {
    for( name in object )
    {
      result += name + '=' + this.objectToString(object[name], 'v') + ';';
    }

    if( result == '' )
      result = '{ -- NO ENUMERABLE PROPERTIES -- }';
    else
      result = '{ ' + result + ' }';
  }

  return result;
}

Logger.prototype.htmlElementToString = function( object )
{
  if( object == undefined )
    return 'UNDEFINED';
  if( object == null )
    return 'NULL';

  return '{ tagName=' + object.tagName
          + (object.id == undefined ? '' : (object.id == null ? '' : '; id=' + object.id))
          + (object.className == undefined ? '' : (object.className == null ? '' : '; className=' + object.className))
          + ' }';
}


// Greate a static instance
Log = new Logger();
setInterval('Log.scrollToLastLine()', 1000);

// Check the logger
Log.info('main', 'Logger starting with mask=' + Log.getLogMask());


/* toolkit/staticexport/StructureEntry.js */
/*
 <DEPENDENCIES>
	 <DEPENDS>Logger</DEPENDS>
 </DEPENDENCIES>
 */
function StructureEntry(cid, url, adaptivesequence)
{
	this.cid = cid;
	this.url = url;
	this.adaptivesequence = adaptivesequence;
	this.children = new Array();
}
StructureEntry.prototype.getContentId = function()
{
	return this.cid;
}
StructureEntry.prototype.getURL = function()
{
	return this.url;
}
StructureEntry.prototype.isAdaptiveSequence = function()
{
	return this.adaptivesequence;
}
StructureEntry.prototype.addChild = function(structureEntry)
{
	this.children.push(structureEntry);
}
StructureEntry.prototype.getChildren = function()
{
	return this.children;
}
StructureEntry.prototype.hasChildren = function()
{
	return (this.children.length > 0);
}
StructureEntry.prototype.setParent = function(structureEntry)
{
	this.parent = structureEntry;
}
StructureEntry.prototype.getParent = function()
{
	return this.parent;
}
StructureEntry.prototype.setIndex = function(index)
{
	this.index = index;
}
StructureEntry.prototype.getIndex = function()
{
	return this.index;
}

/* domapi/Globals.js */
/*
<DEPENDENCIES>
</DEPENDENCIES>
*/
function Point(x,y)
{
  this.x = x;
  this.y = y;
}
Point.prototype.toString = function()
{
  return 'Point{ x=' + this.x + ', y=' + this.y + ' }';
}
function shuffleArray(a)
{
  var aLinearValue = new Array();
  var aLinearUsed = new Array();
  for (var i = 0; i < a.length; i++)
  {
    aLinearValue[i] = a[i];
    aLinearUsed[i] = false;
  }
  var nLow = 0;
  var nHigh = a.length - 1;
  for (var i = 0; i < a.length; i++)
  {
    nRandomIndex = Math.round( (nLow - 0.5) +( ( (nHigh+0.49999) - (nLow-0.5) ) * Math.random() ) );
    var bFoundUnused = false;
    while (!bFoundUnused)
    {
      if (! aLinearUsed[nRandomIndex])
      {
        bFoundUnused = true;
        aLinearUsed[nRandomIndex] = true; 
        break;
      }
      else
      {
        nRandomIndex++;
        if (nRandomIndex > nHigh) nRandomIndex = nLow;
      }
    }
    a[i] = aLinearValue[nRandomIndex];
  }
}

/* common/broker/Broker.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Globals</DEPENDS>
</DEPENDENCIES>
*/
function BrokerActionPipeline()
{
  this.callstack = new Array();
  this.pipeline = new Array();
  this.blockingTasks = new Array();
  this.finalAction = null;
  this.blockCounter = 0;
}
BrokerActionPipeline.prototype.beginAction = function(key)
{
  ;
  this.callstack.push(key);
}
BrokerActionPipeline.prototype.endAction = function(key)
{
  ;
  var closedBrokerKey = this.callstack[this.callstack.length -1];
  if(key != closedBrokerKey)
  {
    ;
  }
  else
  {
    this.callstack.pop();
  }
  this.executePipeline();
}
BrokerActionPipeline.prototype.executePipeline = function()
{
  if((this.callstack.length == 0) && (this.blockCounter == 0))
  {
    this.nextScheduledAction();
  }
}
BrokerActionPipeline.prototype.block = function(key)
{
  ;
  if(!this.blockingTasks[key])
  {
    this.blockingTasks[key] = true;
    this.blockCounter++;
  }
}
BrokerActionPipeline.prototype.unblock = function(key)
{
  ;
  if(this.blockingTasks[key])
  {
    this.blockingTasks[key] = false;
    this.blockCounter--;
  }
  this.executePipeline();
}
BrokerActionPipeline.prototype.scheduleFinalAction = function(action, parameterArray)
{
  ;
  if(this.finalAction)
  {
    ;
  }
  else
  {
    var actionObject = new Object();
    actionObject.action = action;
    actionObject.parameterArray = parameterArray;
    this.finalAction = actionObject;
  }
}
BrokerActionPipeline.prototype.scheduleAction = function(action, parameterArray)
{
  ;
  var actionObject = new Object();
  actionObject.action = action;
  actionObject.parameterArray = parameterArray;
  this.pipeline.push(actionObject);
  this.executePipeline();
}
BrokerActionPipeline.prototype.nextScheduledAction = function()
{
  ;
  if(this.pipeline.length > 0)
  {
    ;
    var actionObject = this.pipeline.shift();
    BrokerActions_callAction(actionObject.action, actionObject.parameterArray);
    this.executePipeline();
  }
  else if(this.finalAction)
  {
    ;
    var callAction = this.finalAction;
    this.finalAction = null;
    BrokerActions_callAction(callAction.action, callAction.parameterArray);
  }
}
function BrokerActions()
{
   this.init()
}
BrokerActions.prototype.init = function(name)
{
  this.actions = new Array();
}
BrokerActions.prototype.callActions = function(parameterArray)
{
  for(id in this.actions)
  {
    var action = this.actions[id];
    BrokerActions_callAction(action, parameterArray);
  }
}
BrokerActions.prototype.addAction = function(action)
{
  this.actions.push(action);
}
function BrokerActions_callAction(action, parameterArray)
{
  if(parameterArray)
  {
    action.invoke(
      parameterArray[0],
      parameterArray[1],
      parameterArray[2],
      parameterArray[3],
      parameterArray[4],
      parameterArray[5],
      parameterArray[6],
      parameterArray[7],
      parameterArray[8]);
  }
  else
  {
    action.invoke();
  }
}
function Broker(name)
{
  this.init(name)
}
Broker.actionPipeline = new BrokerActionPipeline();
Broker.prototype.init = function(name)
{
  this.name = name;
  this.callbacks = new Array();
}
Broker.prototype.getMessage = function(message, parameterArray)
{
  var key = this.name + '_' + message;
  Broker.actionPipeline.beginAction(key);
  var callback = this.callbacks[message];
  if(callback)
  {
    callback.callActions(parameterArray);
  }
  Broker.actionPipeline.endAction(key);
}
Broker.prototype.register = function(match, obj, callback)
{
  ;
}
Broker.prototype.registerAction = function(match, action)
{
  var callback = this.callbacks[match];
  if(!callback)
  {
    callback = new BrokerActions();
    this.callbacks[match] = callback;
  }
  callback.addAction(action);
}

/* actions/Actions.js */
/*
<DEPENDENCIES>
  <DEPENDS>Broker</DEPENDS>
</DEPENDENCIES>
*/
function ChangeImageAction(domId, url)
{
    this.domId = domId;
    this.url = url;
}
ChangeImageAction.prototype.invoke = function()
{
    document.getElementById(this.domId).src=this.url;
}
function SetStyleAction(domId, property, value)
{
    this.domId = domId;
    this.property = property;
    this.value = value;
}
SetStyleAction.prototype.invoke = function()
{
    document.getElementById(this.domId).style[this.property] = this.value;
}
function ObjectMethodAction(objectName, methodname, param1, param2, param3, param4, param5, param6, param7)
{
    this.objectName= objectName;
    this.methodname = methodname;
    this.param1 = param1;
    this.param2 = param2;
    this.param3 = param3;
    this.param4 = param4;
    this.param5 = param5;
    this.param6 = param6;
    this.param7 = param7;
}
ObjectMethodAction.prototype.invoke = function(parameter1, parameter2, parameter3, parameter4, parameter5, parameter6, parameter7)
{
    if (typeof parameter1 != 'undefined')
        return this.objectName[this.methodname](parameter1, parameter2, parameter3, parameter4, parameter5, parameter6, parameter7);
    else
        return this.objectName[this.methodname](this.param1, this.param2, this.param3, this.param4, this.param5, this.param6, this.param7);
}
function ObjectMethodWithParameterAction(objectName, methodname, param1, param2, param3, param4, param5, param6, param7)
{
    this.objectName= objectName;
    this.methodname = methodname;
    this.param1 = param1;
    this.param2 = param2;
    this.param3 = param3;
    this.param4 = param4;
    this.param5 = param5;
    this.param6 = param6;
    this.param7 = param7;
}
ObjectMethodWithParameterAction.prototype.invoke = function(parameter1)
{
    if (typeof parameter1 != 'undefined')
        return this.objectName[this.methodname](parameter1, this.param1, this.param2, this.param3, this.param4, this.param5, this.param6, this.param7);
    else
        return this.objectName[this.methodname](this.param1, this.param2, this.param3, this.param4, this.param5, this.param6, this.param7);
}
function SyncObjectMethodAction(objectName, methodname, param1, param2, param3, param4, param5, param6, param7)
{
    this.objectName= objectName;
    this.methodname = methodname;
    this.param1 = param1;
    this.param2 = param2;
    this.param3 = param3;
    this.param4 = param4;
    this.param5 = param5;
    this.param6 = param6;
    this.param7 = param7;
}
SyncObjectMethodAction.prototype.invoke = function(parameter1, parameter2, parameter3, parameter4, parameter5, parameter6, parameter7)
{
  Broker.actionPipeline.scheduleAction(new ObjectMethodAction(this, "invokeTarget"), [parameter1, parameter2, parameter3, parameter4, parameter5, parameter6, parameter7]);
}
SyncObjectMethodAction.prototype.invokeTarget = function(parameter1, parameter2, parameter3, parameter4, parameter5, parameter6, parameter7)
{
    if (typeof parameter1 != 'undefined')
        return this.objectName[this.methodname](parameter1, parameter2, parameter3, parameter4, parameter5, parameter6, parameter7);
    else
        return this.objectName[this.methodname](this.param1, this.param2, this.param3, this.param4, this.param5, this.param6, this.param7);
}
function FinalObjectMethodAction(objectName, methodname, param1, param2, param3, param4, param5, param6, param7)
{
    this.objectName= objectName;
    this.methodname = methodname;
    this.param1 = param1;
    this.param2 = param2;
    this.param3 = param3;
    this.param4 = param4;
    this.param5 = param5;
    this.param6 = param6;
    this.param7 = param7;
}
FinalObjectMethodAction.prototype.invoke = function(parameter1, parameter2, parameter3, parameter4, parameter5, parameter6, parameter7)
{
  Broker.actionPipeline.scheduleFinalAction(new ObjectMethodAction(this, "invokeTarget"), [parameter1, parameter2, parameter3, parameter4, parameter5, parameter6, parameter7]);
}
FinalObjectMethodAction.prototype.invokeTarget = function(parameter1, parameter2, parameter3, parameter4, parameter5, parameter6, parameter7)
{
    if (typeof parameter1 != 'undefined')
        return this.objectName[this.methodname](parameter1, parameter2, parameter3, parameter4, parameter5, parameter6, parameter7);
    else
        return this.objectName[this.methodname](this.param1, this.param2, this.param3, this.param4, this.param5, this.param6, this.param7);
}
function IgnoreRuntimeParameterAction(objectName, methodname, param1, param2, param3, param4, param5, param6, param7)
{
    this.objectName= objectName;
    this.methodname = methodname;
    this.param1 = param1;
    this.param2 = param2;
    this.param3 = param3;
    this.param4 = param4;
    this.param5 = param5;
    this.param6 = param6;
    this.param7 = param7;
}
IgnoreRuntimeParameterAction.prototype.invoke = function()
{
    return this.objectName[this.methodname](this.param1, this.param2, this.param3, this.param4, this.param5, this.param6, this.param7);
}
function AlertAction(text)
{
    this.text = text;
}
AlertAction.prototype.invoke = function()
{
    alert(this.text);
}
function SolvedAction(objectName, methodname)
{
    this.objectName = objectName;
    this.methodname = methodname;
}
SolvedAction.prototype.invoke = function(para1, para2, para3, para4, para5, para6, para7, para8)
{
    this.object[this.methodname](para1, para2, para3, para4, para5, para6, para7, para8); 
}
function DoNothingAction()
{
}
DoNothingAction.prototype.invoke = function()
{
}

/* toolkit/staticexport/PropertyService.js */
 /*
 <DEPENDENCIES>
	 <DEPENDS>Logger</DEPENDS>
	 <DEPENDS>Actions</DEPENDS>
 </DEPENDENCIES>
 */
 function PropertyService(id)
 {
	this.id = id;
	this.initialized = false;
	this.listener = new Array();
	this.properties = new Array();
 }
 PropertyService.prototype.start = function() {
	if (serviceregistry)
	{
		;
		serviceregistry.registerService('propertyservice', this);
		
		this.initialized = true;
			this.notifyListener();
	}
 }
 PropertyService.prototype.getPropertyValue = function(key)
 {
	return this.properties[key];
 }
 PropertyService.prototype.setPropertyValue = function(key, value)
 {
	this.properties[key] = value;
 }
 PropertyService.prototype.registerLoadedListener= function(action) {
	;
	return this.listener.push(action);
 }
 PropertyService.prototype.notifyListener= function() {
	;
	for (var i = 0; i < this.listener.length; i++)
	{
		this.listener[i].invoke();
	}
 }
 PropertyService.prototype.isInitialized = function() {
	return this.initialized;
 }

/* toolkit/staticexport/LocationService.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Actions</DEPENDS>
</DEPENDENCIES>
*/
function LocationService(id)
{
	this.id = id;
	this.listener = new Array();
	this.initialized = false;
	this.locations = new Array();
}
LocationService.prototype.start = function()
{
	if (serviceregistry)
	{
		;
		serviceregistry.registerService('locationservice', this);
		
		this.initialized = true;
		this.notifyListener();
	}
}
LocationService.prototype.registerLoadedListener = function(action)
{
	;
	return this.listener.push(action);
}
LocationService.prototype.notifyListener = function()
{
	;
	for (var i = 0; i < this.listener.length; i++)
	{
		this.listener[i].invoke();
	}
}
LocationService.prototype.isInitialized = function()
{
	return this.initialized;
}
LocationService.prototype.hasLastVistedLocation = function()
{
	return (this.lastvisted != null);
}
LocationService.prototype.getLastVistedLocation = function()
{
	return this.lastvisted;
}
LocationService.prototype.getStartLocation = function()
{
	return 'start.html';
}
LocationService.prototype.setLastVistedLocation = function(folder, structureEntry)
{
	if (folder && structureEntry)
	{
		this.locations[folder.getContentId()] = new LastVisited(folder, structureEntry);
		this.lastvisted = structureEntry.getURL();
	}
}
LocationService.prototype.setInitialLastVisitedLocation = function(url)
{
	if ('' != url)
		this.lastvisted = url;
}
LocationService.prototype.getLastVistedLocationInFolder = function(folderId)
{
	var lastvisited = this.locations[folderId];
	if (lastvisited)
	{
		return lastvisited.getURL();
	}
	else
	{
		return null;
	}
}
LocationService.prototype.hasLastVistedLocationInFolder = function(folderId)
{
	return (this.getLastVistedLocationInFolder(folderId) != null);
}
LocationService.prototype.getAllLastVisitedLocations = function()
{
	var all = new Array();
	for (var folderId in this.locations)
	{
		all.push(this.locations[folderId]);
	}
	return all;
}
function LastVisited(folder, entry)
{
	this.folder = folder;
	this.entry = entry;
}
LastVisited.prototype.getURL = function()
{
	return this.entry.getURL();
}
LastVisited.prototype.getSerialized = function()
{
	var serialized = this.folder.getIndex() + "_" + this.entry.getIndex();
	return serialized;
}

/* toolkit/staticexport/LMSSuspendDataSerializer.js */
/*
<DEPENDENCIES>
	<DEPENDS>Logger</DEPENDS>
</DEPENDENCIES>
*/
function LMSSuspendDataSerializer(evaluationService, locationService, structureName, structureService)
{
	;
	this.evaluationService = evaluationService;
	this.locationService = locationService;
	this.structureName = structureName;
	this.structureService = structureService;
}
LMSSuspendDataSerializer.prototype.serializeSuspendData = function()
{
	var suspenddata = '';
	
	var entries = this.evaluationService.getAllEvaluationEntries();
	for (var i = 0; i < entries.length; i++)
	{
		var entry = entries[i];
		
		if (entry.hasData())
			suspenddata = suspenddata + 'x' + i + '_' + entry.getData().getSerialized();
	}
	suspenddata = suspenddata + 'z'; 
	var locations = this.locationService.getAllLastVisitedLocations();
	for (var i = 0; i < locations.length; i++)
	{
		var entry = locations[i];
		suspenddata = suspenddata + 'x' + entry.getSerialized();
	}
	return suspenddata;
}
LMSSuspendDataSerializer.prototype.deserializeSuspendData = function(serializeddata)
{
	if (serializeddata && serializeddata != '')
	{
		
		
		serializeddata = serializeddata + '';
		serializeddata = this.trim(serializeddata);
		
		var data = serializeddata.replace(/_/g, ',');
		var data = data.replace(/t/g, 'true');
		var data = data.replace(/f/g, 'false');
		var blocks = data.split('z');
		if (blocks.length == 0)
		{
			
			this.deserializeEvaluationData(data);
		}
		else
		{
			for (var i = 0; i < blocks.length; i++)
			{
				if (i == 0)
				{
					
					this.deserializeEvaluationData(blocks[i]);
				}
				else if (i == 1)
				{
				 	
					this.deserializeLastVisitedLocations(blocks[i]);
				}
			}
		}
	}
}
LMSSuspendDataSerializer.prototype.deserializeEvaluationData = function(serializeddata)
{
	if (serializeddata && serializeddata != '')
	{
		var evalentries = serializeddata.split('x');
		for (var i = 0; i < evalentries.length; i++)
		{
			try
			{
				if (evalentries[i] != '')
					eval('this.createResultData(' + evalentries[i] + ');');
			}
			catch (e)
			{
				
			}
		}
	}
}
LMSSuspendDataSerializer.prototype.deserializeLastVisitedLocations = function(serializeddata)
{
	if (serializeddata && serializeddata != '')
	{
		var entries = serializeddata.split('x');
		for (var i = 0; i < entries.length; i++)
		{
			try
			{
				if (entries[i] != '')
					eval('this.createLastVisitedLocation(' + entries[i] + ');');
			}
			catch (e)
			{
				
			}
		}
	}
}
LMSSuspendDataSerializer.prototype.createResultData = function(index, trace, numCorrect, numCorrectStatus, numWrongStatus, numTouched,
																	numTotal, numWrong,
																	numCorrectSystem, numWrongSystem)
{
	var evalentry = this.evaluationService.getEvaluationEntryByIndex(index);
	if (evalentry)
	{
		var resultdata = null;
		if (trace)
		{
			resultdata = new ResultData(trace, numTouched, numTotal, numCorrect, numWrong,
					numCorrectStatus, numWrongStatus,
					numCorrectSystem, numWrongSystem);
		}
		else
		{
			resultdata = new ResultData(trace, 0, 0, numCorrect, 0,
					numCorrectStatus, numWrongStatus, 0, 0);
		}
		evalentry.setData(resultdata);
	}
}
LMSSuspendDataSerializer.prototype.createLastVisitedLocation = function(folderIndex, entryIndex)
{
	var folder = this.structureService.getStructureEntryByIndex(this.structureName, folderIndex);
	if (folder)
	{
		var entry = this.structureService.getStructureEntryByIndex(this.structureName, entryIndex);
		if (entry)
			this.locationService.setLastVistedLocation(folder, entry);
	}
}
LMSSuspendDataSerializer.prototype.trim = function(inputString) {
   if (typeof inputString != "string") { return inputString; }
   var retValue = inputString;
   var ch = retValue.substring(0, 1);
   while (ch == " ") { 
      retValue = retValue.substring(1, retValue.length);
      ch = retValue.substring(0, 1);
   }
   ch = retValue.substring(retValue.length-1, retValue.length);
   while (ch == " ") { 
      retValue = retValue.substring(0, retValue.length-1);
      ch = retValue.substring(retValue.length-1, retValue.length);
   }
   while (retValue.indexOf("  ") != -1) { 
      retValue = retValue.substring(0, retValue.indexOf("  ")) + retValue.substring(retValue.indexOf("  ")+1, retValue.length); 
   }
   return retValue; 
}

/* toolkit/staticexport/LMSConnectorManagerStandalone.js */
/*
	<DEPENDENCIES>
		<DEPENDS>Logger</DEPENDS>
		<DEPENDS>Actions</DEPENDS>
		<DEPENDS>LocationService</DEPENDS>
		<DEPENDS>LMSSuspendDataSerializer</DEPENDS>
	</DEPENDENCIES>
	*/
function LMSConnectorManagerStandalone()
{
	;
	this.suspendDataSerializer = null;
	this.startDate = 0;
	this.structureName = null;
	this.achievementService = null;
	this.structureService = null;
	this.evaluationService = null;
	this.locationService = null;
	this.lastVisitedLocation = 0;
	
	this.score = new ScoreStatistics(0, 0, 0, 0, 1);
	this.sessiontime = '00:00:00.0';
	this.finished = false;
	
}
LMSConnectorManagerStandalone.prototype.start = function()
{
	
	serviceregistry.registerService('lmsconnector', this);
}
LMSConnectorManagerStandalone.prototype.initConnector = function()
{
	if (this.locationService.isInitialized()
			&& this.achievementService.isInitialized()
			&& this.propertyService.isInitialized()
			&& this.linkService.isInitialized()
			)
	{
		
		this.startDate = new Date().getTime();
		
		this.startWBT();
	}
}
LMSConnectorManagerStandalone.prototype.startWBT = function()
{
	var startloc = this.locationService.getStartLocation();
	parent.frames['mainframe'].location.replace(startloc);
}
LMSConnectorManagerStandalone.prototype.setStructureName = function(structureName)
{
	;
	this.structureName = structureName;
}
LMSConnectorManagerStandalone.prototype.setAchievementService = function(achievementService)
{
	;
	this.achievementService = achievementService;
	
	this.achievementService.registerLoadedListener(new ObjectMethodAction(this, "initConnector"));
}
LMSConnectorManagerStandalone.prototype.setStructureService = function(structureService)
{
	;
	this.structureService = structureService;
}
LMSConnectorManagerStandalone.prototype.setEvaluationService = function(evaluationService)
{
	;
	this.evaluationService = evaluationService;
}
LMSConnectorManagerStandalone.prototype.setLocationService = function(locationService)
{
	;
	this.locationService = locationService;
	
	this.locationService.registerLoadedListener(new ObjectMethodAction(this, "initConnector"));
}
LMSConnectorManagerStandalone.prototype.setPropertyService = function(propertyService)
{
	;
	this.propertyService = propertyService;
	
	this.propertyService.registerLoadedListener(new ObjectMethodAction(this, "initConnector"));
}
LMSConnectorManagerStandalone.prototype.setLinkService = function(linkService)
{
	;
	this.linkService = linkService;
	
	this.linkService.registerLoadedListener(new ObjectMethodAction(this, "initConnector"));
}
LMSConnectorManagerStandalone.prototype.finishConnection = function()
{
	;
	
}
LMSConnectorManagerStandalone.prototype.notifyFinished = function(timeout)
{
	;
	this.finished = true;
}
LMSConnectorManagerStandalone.prototype.retrieveAchievementData = function()
{
	;
}
LMSConnectorManagerStandalone.prototype.setLastVisitedLocation= function(folderId, entryId) {
	if (entryId)
	{
		var folder =
		        this.structureService.getStructureEntry(this.structureName, folderId);
		var structureEntry =
				this.structureService.getStructureEntry(this.structureName, entryId);
		if (folder && structureEntry)
		{
			this.locationService.setLastVistedLocation(folder, structureEntry);
		}
		else if (structureEntry)
		{
			
			this.locationService.setInitialLastVisitedLocation(structureEntry.getURL());
		}
	}
}
LMSConnectorManagerStandalone.prototype.setInitialLastVisitedLocation = function(url)
{
	if (url)
	{
		this.locationService.setInitialLastVisitedLocation(url);
	}
}
LMSConnectorManagerStandalone.prototype.sendData = function()
{
	
}
LMSConnectorManagerStandalone.prototype.getSuspendDataSerializer = function()
{
	if (this.suspendDataSerializer == null)
		this.suspendDataSerializer =
				new LMSSuspendDataSerializer(this.evaluationService, this.locationService, this.structureName, this.structureService);
	return this.suspendDataSerializer;
}
LMSConnectorManagerStandalone.prototype.serializeSuspendData = function()
{
	return this.getSuspendDataSerializer().serializeSuspendData();
}
LMSConnectorManagerStandalone.prototype.deserializeSuspendData = function(serializeddata)
{
	this.getSuspendDataSerializer().deserializeSuspendData(serializeddata);
}
LMSConnectorManagerStandalone.prototype.computeTime = function()
{
	if (this.startDate != 0)
	{
		var currentDate = new Date().getTime();
		var elapsedSeconds = (currentDate - this.startDate) / 1000;
		var formattedTime = this.convertTotalSeconds(elapsedSeconds);
	}
	else
	{
		formattedTime = "00:00:00.0";
	}
	return formattedTime;
}
LMSConnectorManagerStandalone.prototype.convertTotalSeconds = function (ts)
{
	var sec = (ts % 60);
	ts -= sec;
	var tmp = (ts % 3600);
	
	ts -= tmp;
	
	
	sec = Math.round(sec * 100) / 100;
	var strSec = new String(sec);
	var strWholeSec = strSec;
	var strFractionSec = "";
	if (strSec.indexOf(".") != -1)
	{
		strWholeSec = strSec.substring(0, strSec.indexOf("."));
		strFractionSec = strSec.substring(strSec.indexOf(".") + 1, strSec.length);
	}
	if (strWholeSec.length < 2)
	{
		strWholeSec = "0" + strWholeSec;
	}
	strSec = strWholeSec;
	if (strFractionSec.length)
	{
		strSec = strSec + "." + strFractionSec;
	}
	if ((ts % 3600) != 0)
		var hour = 0;
	else var hour = (ts / 3600);
	if ((tmp % 60) != 0)
		var min = 0;
	else var min = (tmp / 60);
	if ((new String(hour)).length < 2)
		hour = "0" + hour;
	if ((new String(min)).length < 2)
		min = "0" + min;
	var rtnVal = hour + ":" + min + ":" + strSec;
	return rtnVal;
}
LMSConnectorManagerStandalone.prototype.trim = function(inputString)
{
	
	
	
	if (typeof inputString != "string")
	{
		return inputString;
	}
	var retValue = inputString;
	var ch = retValue.substring(0, 1);
	while (ch == " ")
	{ 
		retValue = retValue.substring(1, retValue.length);
		ch = retValue.substring(0, 1);
	}
	ch = retValue.substring(retValue.length - 1, retValue.length);
	while (ch == " ")
	{ 
		retValue = retValue.substring(0, retValue.length - 1);
		ch = retValue.substring(retValue.length - 1, retValue.length);
	}
	while (retValue.indexOf("  ") != -1)
	{ 
		retValue = retValue.substring(0, retValue.indexOf("  ")) + retValue.substring(retValue.indexOf("  ") + 1, retValue.length);
		
	}
	return retValue;
	
}

/* toolkit/staticexport/ResultData.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
</DEPENDENCIES>
*/
 function ResultData(trace, numTouched, numTotal, numCorrect, numWrong,
            numCorrectStatus, numWrongStatus,
            numCorrectSystem, numWrongSystem)
            {
   this.trace = trace;
   this.numTouched = numTouched;
        this.numTotal = numTotal;
        this.numCorrect = numCorrect;
        this.numWrong = numWrong;
        this.numCorrectStatus = numCorrectStatus;
        this.numWrongStatus = numWrongStatus;
        this.numCorrectSystem = numCorrectSystem;
        this.numWrongSystem = numWrongSystem;
}
ResultData.prototype.getNumTouched= function() {
	return this.numTouched;
}
ResultData.prototype.getNumTotal= function() {
	return this.numTotal;
}
ResultData.prototype.getNumCorrect= function() {
	return this.numCorrect;
}
ResultData.prototype.getNumWrong= function() {
	return this.numWrong;
}
ResultData.prototype.getNumCorrectStatus= function() {
	return this.numCorrectStatus;
}
ResultData.prototype.getNumWrongStatus= function() {
	return this.numWrongStatus;
}
ResultData.prototype.getNumCorrectSystem= function() {
	return this.numCorrectSystem;
}
ResultData.prototype.getNumWrongSystem= function() {
	return this.numWrongSystem;
}
ResultData.prototype.isTraceData= function() {
	return this.trace;
}
ResultData.prototype.getSerialized= function() {
	var serialized = '';
	if (this.trace)	
	{
		serialized =
		"t_"
	    + this.numCorrect + "_"
	    + this.numCorrectStatus + "_"
	    + this.numWrongStatus + "_"
	    + this.numTouched + "_"
	    + this.numTotal + "_"
	    + this.numWrong + "_"
	    + this.numCorrectSystem + "_"
	    + this.numWrongSystem;
	}
	else	
	{
		serialized =
		"f_"
	    + this.numCorrect + "_"
	    + this.numCorrectStatus + "_"
	    + this.numWrongStatus;
	}
	return serialized;
}

/* toolkit/staticexport/EvaluationEntry.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>ResultData</DEPENDS>
</DEPENDENCIES>
*/
function EvaluationEntry(cid, trace)
{
  	this.cid = cid;
  	this.trace = trace;
  	this.resultdata = null;
}
EvaluationEntry.prototype.getContentId= function() {
	return this.cid ;
}
EvaluationEntry.prototype.hasData= function() {
	return (this.resultdata);
}
EvaluationEntry.prototype.hasTraceData= function() {
	if (this.resultdata)
	{
		return this.resultdata.isTraceData();
	}
	else
		return false;
}
EvaluationEntry.prototype.hasTrackData= function() {
	return (this.resultdata);
}
EvaluationEntry.prototype.setData= function(resultdata) {
	this.resultdata = resultdata;
}
EvaluationEntry.prototype.getData= function() {
	return this.resultdata;
}
EvaluationEntry.prototype.isTrace= function() {
	return this.trace;
}

/* toolkit/staticexport/EvaluationManager.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>EvaluationEntry</DEPENDS>
</DEPENDENCIES>
*/
function EvaluationManager(id)
{
  	this.id = id;
  	this.entries = new Array();
  	this.entriesByIndex = new Array();
  	this.indexcounter = 0;
}
EvaluationManager.prototype.addEvaluationEntry= function(evaluationEntry) {
	this.entries[evaluationEntry.getContentId()] = evaluationEntry;
	this.indexcounter = this.entriesByIndex.push(evaluationEntry);
}
EvaluationManager.prototype.getEvaluationEntry= function(cid) {
	return this.entries[cid];
}
EvaluationManager.prototype.getEvaluationEntryByIndex= function(index) {
	return this.entriesByIndex[index];
}
EvaluationManager.prototype.getAllEvaluationEntries= function() {
	return this.entriesByIndex;
}

/* toolkit/staticexport/StructureManager.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
</DEPENDENCIES>
*/
function StructureManager(id)
{
  	this.id = id;
  	this.structures = new Array();
	this.structuresByIndex = new Array();
	this.structureNames = new Array();
  	this.structureRoots = new Array();
	this.structureCount = 0;
}
StructureManager.prototype.start= function() {
	
	for (var i = 0; i <  this.structureNames.length; i++) {
    	var structure = this.structures[this.structureNames[i]];
    	if (structure)
    	{
    		for (var se in structure) {
    			var structureEntry = structure[se];
    			if (structureEntry && !structureEntry.getParent())
    			{
    				
    				this.structureRoots[this.structureNames[i]] = structureEntry;
    				break;
    			}
    		}
    	}
	}
}
StructureManager.prototype.addStructureEntry= function(structurename, structureEntry) {
	var structure = this.structures[structurename]; 
	var structureByIndex = this.structuresByIndex[structurename];
	if (!structure)
	{
		structure = new Array();
		this.structures[structurename] = structure;
		structureByIndex = new Array();
		this.structuresByIndex[structurename] = structureByIndex;
		this.structureNames.push(structurename);
	}
	structure[structureEntry.getContentId()] = structureEntry;
	structureByIndex[this.structureCount] = structureEntry;
	structureEntry.setIndex(this.structureCount);
	this.structureCount++;
}
StructureManager.prototype.getStructureEntry= function(structurename, cid) {
	var structure = this.structures[structurename]; 
	if (!structure)
		return null;
	else
		return (this.structures[structurename])[cid];
}
StructureManager.prototype.getRoot= function(structurename) {
	return this.structureRoots[structurename];
}
StructureManager.prototype.getStructureEntry= function(structurename, cid) {
	var structure = this.structures[structurename]; 
	if (!structure)
		return null;
	else
		return (this.structures[structurename])[cid];
}
StructureManager.prototype.getStructureEntryByIndex= function(structurename, index) {
	var structure = this.structuresByIndex[structurename]; 
	if (!structure)
		return null;
	else
		return structure[index];
}

/* toolkit/staticexport/LinkService.js */
/*
<DEPENDENCIES>
  <DEPENDS>Logger</DEPENDS>
  <DEPENDS>Actions</DEPENDS>
</DEPENDENCIES>
*/
function LinkInfo(cid, title, description, value)
{
	this.cid = cid;
	this.title = title;
	this.description = description;
	this.value = value;
}
LinkInfo.prototype.getId = function()
{
	return this.cid;
}
LinkInfo.prototype.getValue = function()
{
	return this.value;
}
LinkInfo.prototype.setValue = function(value)
{
	this.value = value;
}
function LinkParser()
{
	this.parserValues = new Array();
}
LinkParser.prototype.setParserValue = function(valuekey, value)
{
	this.parserValues[valuekey] = value;
}
LinkParser.prototype.parse = function(value)
{
	
	return value;
}
function LinkService(id)
{
	this.id = id;
	this.listener = new Array();
	this.initialized = false;
	this.links = new Array();
	this.parser = new LinkParser();
}
LinkService.prototype.start = function() {
	
	this.initialized = true;
	serviceregistry.registerService('linkservice', this);
	this.notifyListener();
}
LinkService.prototype.addLink = function(type, id, title, description, value)
{
	;
	var link = new LinkInfo(id, title, description, value);
	this.links[id] = link;
}
LinkService.prototype.setParserValue = function(valuekey, value) {
	this.parser.setParserValue(valuekey, value);
}
LinkService.prototype.getLink = function(id)
{
	;
	var link = this.links[id];
	if (link)
	{
		;
		return this.parser.parse(link.getValue());
	}
	return null;
}
LinkService.prototype.registerLoadedListener= function(action) {
	;
	return this.listener.push(action);
}
LinkService.prototype.notifyListener= function() {
	;
	for (var i = 0; i < this.listener.length; i++)
	{
		this.listener[i].invoke();
	}
}
LinkService.prototype.isInitialized = function() {
	return this.initialized;
}

/* toolkit/staticexport/ScoreStatistics.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
</DEPENDENCIES>
*/
function ScoreStatistics(fractionCorrect, fractionCorrectWithHelp, fractionWrong, fractionDone, fractionUndone) {
  	;
  	this.fractionCorrect = fractionCorrect;
  	this.fractionCorrectWithHelp = fractionCorrectWithHelp;
  	this.fractionWrong = fractionWrong;
  	this.fractionDone = fractionDone;
  	this.fractionUndone = fractionUndone;
}
ScoreStatistics.prototype.getFractionCorrect= function() {
	;
	return this.fractionCorrect;
}
ScoreStatistics.prototype.getFractionCorrectWithHelp= function() {
	;
	return this.fractionCorrectWithHelp;
}
ScoreStatistics.prototype.getFractionWrong= function() {
	;
	return this.fractionWrong;
}
ScoreStatistics.prototype.getFractionDone= function() {
	;
	return this.fractionDone;
}
ScoreStatistics.prototype.getFractionUndone= function() {
	;
	return this.fractionUndone;
}

/* toolkit/staticexport/ScoreEvaluatorStandard.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>ScoreStatistics</DEPENDS>
    <DEPENDS>EvaluationEntry</DEPENDS>
</DEPENDENCIES>
*/
function ScoreEvaluatorStandard() {
  	;
}
ScoreEvaluatorStandard.prototype.evaluateSingleScoreTrace = function(evaluationEntry)
{
	if (!evaluationEntry.hasData())
    		return new ScoreStatistics(0,0,0,0,1);
       	else
       	{
		var score = evaluationEntry.getData();	
		
		
		
		
		
		var base = Math.max(score.getNumWrong(), score.getNumCorrect());
		
		
		
		var wrongUser = this.getWrongByUser(score.getNumWrongStatus());
		
		
		
		
		
		
		
		
		var correctOffset = this.getCorrectOffset(score.getNumCorrect(), score.getNumWrong(), wrongUser);
		
		
		
		
		
	    
		
		
		
		
		var correctHelp = this.getCorrectWithHelp(score.getNumCorrectSystem(), base, wrongUser);
		
		
		
		
		var correctUser = this.getCorrectByUser(correctOffset, correctHelp, score.getNumCorrectStatus());
		
		
		
		
		var untouched = this.getUntouched(base, correctUser, wrongUser, correctHelp);
		wrongUser = wrongUser + untouched;
		var fractionCorrectByUser = correctUser / base;
		var fractionCorrectWithHelp = correctHelp / base;
		var fractionWrong = wrongUser / base;
		var fractUntouched = 0;
		var fractUntouched = this.harmonize(
			fractUntouched, fractionWrong,
			fractionCorrectWithHelp, fractionCorrectByUser);
		var fractTracked = 1.0
			- fractUntouched
			- fractionWrong
			- fractionCorrectWithHelp
			- fractionCorrectByUser;
		var fractTracked = Math.max(0.0, fractTracked);
		var stat = new ScoreStatistics(
					fractionCorrectByUser, fractionCorrectWithHelp, fractionWrong,
					fractTracked, fractUntouched);
		return stat;
    	}
}
ScoreEvaluatorStandard.prototype.evaluateSingleScoreTrack = function(evaluationEntry)
{
	if (!evaluationEntry.hasData())
       		return new ScoreStatistics(0, 0, 0, 0, 1);
       	else
        	return new ScoreStatistics(0, 0, 0, 1, 0);
}
ScoreEvaluatorStandard.prototype.evaluateAverage = function(numuntouched, statisticEntries)
{
	var numStats = statisticEntries.length;
	var num = numStats + numuntouched;
	var numTraced = 0;
	var correctByUser = 0;
	var correctWithHelp = 0;
	var wrong = 0;
	var tracked = 0;
	var untouched = 0;
	for (var i = 0; i <  statisticEntries.length; i++) {
		var stat = statisticEntries[i];
		correctByUser += stat.getFractionCorrect();
		correctWithHelp += stat.getFractionCorrectWithHelp();
		wrong += stat.getFractionWrong();
		tracked += stat.getFractionDone();
		untouched += stat.getFractionUndone();
   }
	var fractCorrectByUser = 0;
	var fractCorrectWithHelp = 0;
	var fractWrong = 0;
	var fractTracked = 0;
	var fractUntouched = 1;
	if (num != 0)
	{
		fractCorrectByUser = correctByUser / num;
		fractCorrectWithHelp = correctWithHelp / num;
		fractWrong = wrong / num;
		fractTracked = (tracked) / num;
		fractUntouched = (untouched + numuntouched) / num;
	}
	stat = new ScoreStatistics(
			fractCorrectByUser,
			fractCorrectWithHelp,
			fractWrong,
			fractTracked,
			fractUntouched);
	return stat;
}
ScoreEvaluatorStandard.prototype.getCorrectOffset = function(numCorrect, numWrong, wrongUser)
{
	var offset = 0;
	offset = numWrong - numCorrect - wrongUser;
	if (offset < 0)
		offset = 0;
	return offset;
}
ScoreEvaluatorStandard.prototype.getUntouched = function(base, correctUser, wrongUser, correctHelp)
{
	var untouched = (base - (correctUser + wrongUser + correctHelp));
	if (untouched < 0)
		untouched = 0;
	return untouched;
}
ScoreEvaluatorStandard.prototype.getWrongByUser = function(numWrongStatus)
{
	if(numWrongStatus < 0)
		numWrongStatus = 0;
	return numWrongStatus;
}
ScoreEvaluatorStandard.prototype.getCorrectWithHelp = function(numCorrectSystem, base, wrongUser)
{
	if (numCorrectSystem < 0)
		return 0;
	var correctWithHelp = numCorrectSystem;
	var possibleCorrect = base - wrongUser;
    if (correctWithHelp > possibleCorrect)
    	correctWithHelp = possibleCorrect;
	return correctWithHelp;
}
ScoreEvaluatorStandard.prototype.getCorrectByUser = function(correctOffset, correctHelp, numCorrectStatus)
{
	var correct = 0;
	if (numCorrectStatus < 0)
		numCorrectStatus = 0;
	
    correct = numCorrectStatus + correctOffset - correctHelp;
    if (correct < 0)
   		correct = 0;
    return correct;
}
ScoreEvaluatorStandard.prototype.harmonize = function(fractUntouched, fractionWrong, fractionCorrectWithHelp, fractionCorrectByUser)
{
	sum = fractionWrong
			+ fractionCorrectWithHelp + fractionCorrectByUser;
	if (fractUntouched + sum <= 1)
		return fractUntouched;
}

/* toolkit/staticexport/ServiceRegistry.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
</DEPENDENCIES>
*/
function ServiceRegistry()
{
  	this.services = new Array();
}
var serviceregistry = new ServiceRegistry();
ServiceRegistry.prototype.getService= function(servicename) {
	;
	return this.services[servicename];
}
ServiceRegistry.prototype.registerService= function(servicename, service) {
	;
	this.services[servicename] = service;
}

/* toolkit/staticexport/AchievementService.js */
/*
<DEPENDENCIES>
   <DEPENDS>Logger</DEPENDS>
   <DEPENDS>ScoreEvaluatorStandard</DEPENDS>
   <DEPENDS>ServiceRegistry</DEPENDS>
</DEPENDENCIES>
*/
function AchievementService(id) {
	;
	this.id = id;
	this.scoreStrategy = new ScoreEvaluatorStandard();
	this.numuntouched = 0;
	this.listener = new Array();
  	this.initialized = false;
}
AchievementService.prototype.start = function() {
	if (serviceregistry)
	{
		;
		serviceregistry.registerService('achievementservice', this);
		
		this.initialized = true;
    		this.notifyListener();
	}
}
AchievementService.prototype.setStructureManager = function(structuremanager) {
	;
this.structuremanager = structuremanager;
}
AchievementService.prototype.setEvaluationManager = function(evaluationmanager) {
	;
this.evaluationmanager = evaluationmanager;
}
AchievementService.prototype.getScoreStatisticsTrace = function(structurename, cid) {
	var rootStructureEntry = this.structuremanager.getStructureEntry(structurename,cid); 
	if (!rootStructureEntry)
		return null;
	this.numuntouched = 0;
	var evaluationEntries = new Array();
    this.retrieveEvaluationEntriesTrace(rootStructureEntry, evaluationEntries);
	var staticsEntries = new Array();
	for (var i = 0; i <  evaluationEntries.length; i++) {
		var es = this.scoreStrategy.evaluateSingleScoreTrace(evaluationEntries[i]); 
		staticsEntries.push(es);
   }
   var averageStatistics = this.scoreStrategy.evaluateAverage(this.numuntouched, staticsEntries);
   return averageStatistics;
}
AchievementService.prototype.getScoreStatisticsTraceSingleLevel = function(structurename, cid) {
	var rootStructureEntry = this.structuremanager.getStructureEntry(structurename,cid); 
	if (!rootStructureEntry)
		return null;
	this.numuntouched = 0;
	var evaluationEntries = new Array();
    this.retrieveEvaluationEntriesTraceSingleLevel(rootStructureEntry, evaluationEntries);
	var staticsEntries = new Array();
	for (var i = 0; i <  evaluationEntries.length; i++) {
		var es = this.scoreStrategy.evaluateSingleScoreTrace(evaluationEntries[i]); 
		staticsEntries.push(es);
   }
   var averageStatistics = this.scoreStrategy.evaluateAverage(this.numuntouched, staticsEntries);
   return averageStatistics;
}
AchievementService.prototype.getScoreStatisticsTrack = function(structurename, cid) {
	var rootStructureEntry = this.structuremanager.getStructureEntry(structurename,cid); 
	if (!rootStructureEntry)
		return null;
	this.numuntouched = 0;
	var evaluationEntries = new Array();
    this.retrieveEvaluationEntriesTrack(rootStructureEntry, evaluationEntries);
	var staticsEntries = new Array();
	for (var i = 0; i <  evaluationEntries.length; i++) {
		var es = this.scoreStrategy.evaluateSingleScoreTrack(evaluationEntries[i]); 
		staticsEntries.push(es);
   }
   var averageStatistics = this.scoreStrategy.evaluateAverage(this.numuntouched, staticsEntries);
   return averageStatistics;
}
AchievementService.prototype.retrieveEvaluationEntriesTrace = function(structureEntry, evaluationEntries)
{
	var evalEntry = this.evaluationmanager.getEvaluationEntry(structureEntry.getContentId());
	if (evalEntry && evalEntry.isTrace())
	{
			
			if (evalEntry.hasData())
				evaluationEntries.push(evalEntry);
			else
				this.numuntouched++;
	}
	if (structureEntry.isAdaptiveSequence())
	{
		var numuntouchedbefore = this.numuntouched;
		var evaluationEntriescountbefore = evaluationEntries.length;
		this.retrieveEvaluationEntriesTraceAdaptiveSequence(structureEntry, evaluationEntries);
		if ((evaluationEntries.length == evaluationEntriescountbefore) && (numuntouchedbefore == this.numuntouched))
		{
			
			
			
			var numberofentries = this.retrieveAdaptiveSequenceDefaultPathlength(structureEntry);
			this.numuntouched = this.numuntouched + numberofentries;
            ;
		}
	}
	else if (structureEntry.hasChildren())
	{
		var children = structureEntry.getChildren();
		for (var i = 0; i <  children.length; i++) {
			this.retrieveEvaluationEntriesTrace(children[i], evaluationEntries);
		}
	}
}
AchievementService.prototype.retrieveAdaptiveSequenceDefaultPathlength = function(structureEntry)
{
	var myuntouched = 0;
	if (structureEntry.hasChildren())
	{
    	var fistsubsequence = false;
    	var children = structureEntry.getChildren();
    	for (var i = 0; i < children.length; i++)
    	{
    		var entry = children[i];
    		 ;
    		if (entry.hasChildren())
    		{
    			if (!fistsubsequence)
    			{
    				fistsubsequence = true;
    				myuntouched = myuntouched + this.retrieveAdaptiveSequenceDefaultPathlength(entry);
    			}
    		}
    		else
    		{
    			var evalEntry = this.evaluationmanager.getEvaluationEntry(entry.getContentId());
				if (evalEntry && evalEntry.isTrace())
				{
					
					myuntouched++;
				}
    		}
    	}
	}
	return myuntouched;
}
AchievementService.prototype.retrieveEvaluationEntriesTraceSingleLevel = function(structureEntry, evaluationEntries)
{
	if (structureEntry.hasChildren())
	{
		var children = structureEntry.getChildren();
		for (var i = 0; i <  children.length; i++) {
			var entry = children[i];
			if (!entry.hasChildren())
			{
			 	var evalEntry = this.evaluationmanager.getEvaluationEntry(entry.getContentId());
				if (evalEntry && evalEntry.isTrace())
				{
						
						if (evalEntry.hasData())
							evaluationEntries.push(evalEntry);
						else
							this.numuntouched++;
				}
			}
		}
	}
}
AchievementService.prototype.retrieveEvaluationEntriesTraceAdaptiveSequence = function(structureEntry, evaluationEntries)
{
	if (structureEntry.hasChildren())
	{
		var myEvaluationEntries = new Array();
		var subsequences = new Array();
		var untouched = 0;
		var children = structureEntry.getChildren();
		for (var i = 0; i <  children.length; i++)
		{
			var entry = children[i];
			if (entry.isAdaptiveSequence())
			{
				subsequences.push(entry);
			}
			else
			{
				var evalEntry = this.evaluationmanager.getEvaluationEntry(entry.getContentId());
				if (evalEntry && evalEntry.isTrace())
				{
					
					if (evalEntry.hasData())
						myEvaluationEntries.push(evalEntry);
					else
						untouched++;
				}
			}
		}
		if (untouched > 0 && (children.length - subsequences.length) == untouched)
		{
			
            ;
            ;
            ;
            ;
		}
		else
		{
			var tempEvaluationEntries = new Array();
			var tempTouched = 0;
			for (var i = 0; i <  subsequences.length; i++)
			{
				var subEvaluationEntries = new Array();
				var subTouched = 0;
				;
				this.retrieveEvaluationEntriesTraceAdaptiveSequence(subsequences[i], subEvaluationEntries);
			    if (subEvaluationEntries.length == 0 && subTouched == 0)
			    {
			     	
			    }
			    else
			    {
			    	for (var j = 0; j <  subEvaluationEntries.length; j++)
			    	{
			    		tempEvaluationEntries.push(subEvaluationEntries[j]);
			    	}
			    	tempTouched = tempTouched + subTouched;
			    }
			}
			if (subsequences.length > 0 && (tempEvaluationEntries.length > 0 || tempTouched > 0))
			{
				for (var i = 0; i <  tempEvaluationEntries.length; i++)
				{
					myEvaluationEntries.push(tempEvaluationEntries[i]);
				}
				untouched = untouched + tempTouched;
			}
			else if (subsequences.length > 0)
			{
			 	
            	untouched = 0;
            	myEvaluationEntries = new Array();
			}
			else
			{
			    
			}
			for (var i = 0; i <  myEvaluationEntries.length; i++)
			{
				evaluationEntries.push(myEvaluationEntries[i]);
			}
			this.numuntouched = this.numuntouched + untouched;
			;
			;
		}
	}
}
AchievementService.prototype.retrieveEvaluationEntriesTrack = function(structureEntry, evaluationEntries)
{
	var evalEntry = this.evaluationmanager.getEvaluationEntry(structureEntry.getContentId());
	if (evalEntry)
	{
			
			if (evalEntry.hasData())
				evaluationEntries.push(evalEntry);
			else
				this.numuntouched++;
	}
	if (structureEntry.hasChildren())
	{
		var children = structureEntry.getChildren();
		for (var i = 0; i <  children.length; i++) {
			this.retrieveEvaluationEntriesTrack(children[i], evaluationEntries);
		}
	}
}
AchievementService.prototype.setTraceData = function(cid, nTouched, nTotal, nCorrect, nWrong,
nCorrectStatus, nWrongStatus, nCorrectSystem, nWrongSystem)
{
	;
	var evalEntry = this.evaluationmanager.getEvaluationEntry(cid);
	if (evalEntry)
	{
		var resultData = new ResultData(true, nTouched,nTotal,nCorrect,nWrong,nCorrectStatus, nWrongStatus, nCorrectSystem, nWrongSystem);
		evalEntry.setData(resultData);
	}
}
AchievementService.prototype.setTrackData = function(cid, nTouched, nTotal, nCorrect, nWrong,
nCorrectStatus, nWrongStatus, nCorrectSystem, nWrongSystem)
{
	;
	var evalEntry = this.evaluationmanager.getEvaluationEntry(cid);
	if (evalEntry)
	{
		var trackData = new ResultData(false, nTouched,nTotal,nCorrect,nWrong,nCorrectStatus, nWrongStatus, nCorrectSystem, nWrongSystem);
		evalEntry.setData(trackData);
	}
}
AchievementService.prototype.clearResults = function(cid)
{
	;
	var rootStructureEntry = this.structuremanager.getStructureEntry('lrn',cid); 
	if (!rootStructureEntry)
		return null;
	var evaluationEntries = new Array();
    this.retrieveEvaluationEntriesTrace(rootStructureEntry, evaluationEntries);
	for (var i = 0; i <  evaluationEntries.length; i++) {
		evaluationEntries[i].setData(null);
   }
}
AchievementService.prototype.startTest = function(cid)
{
	;
    this.clearResults(cid);
}
AchievementService.prototype.finishTest = function()
{
	;
}
AchievementService.prototype.registerLoadedListener= function(action) {
	;
	return this.listener.push(action);
}
AchievementService.prototype.notifyListener= function() {
	;
	for (var i = 0; i < this.listener.length; i++)
	{
		this.listener[i].invoke();
	}
}
AchievementService.prototype.isInitialized = function() {
	return this.initialized;
}
